Option Explicit On 
Option Strict On
Option Compare Binary

Imports System
Imports System.Drawing
Imports System.Threading
Imports System.Windows.Forms

' <remarks>
'   Erweitert die Standard-ListBox dahingehend, 
'   dass darin alle verfgbaren Schriftarten angezeigt werden.
' </remarks>
Public Class FontListBox
    Inherits ListBox

    ' <summary>
    '   Wird ausgelst, wenn alle Schriftarten 
    '   in das Steuerelement geladen wurden.
    ' </summary>
    ' <param name="sender">Objekt, das das Ereignis auslst.</sender>
    ' <param name="e">Mit dem Ereignis verbundene Informationen.</sender>
    Public Event EnumerationFinished( _
      ByVal sender As Object, _
      ByVal e As EnumerationFinishedEventArgs)

    Private m_EnumThread As Thread
    Private m_blnHasFocus As Boolean

    Private m_sngFontSize As Single
    Private m_blnEnumerating As Boolean

    Private m_intValidFontsCount As Integer
    Private m_intInvalidFontsCount As Integer

#Region " Vom Windows Form Designer generierter Code "
    Public Sub New()
        MyBase.New()
        InitializeComponent()

        ' Vorbereiten.
        Me.DrawMode = DrawMode.OwnerDrawVariable
        Me.FontSize = Me.DefaultFont.Size
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private components As System.ComponentModel.IContainer

    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub InitializeComponent()
        components = New System.ComponentModel.Container()

    End Sub
#End Region

    Private Sub FontListBox_MeasureItem( _
      ByVal sender As Object, _
      ByVal e As MeasureItemEventArgs) _
      Handles MyBase.MeasureItem

        ' Es gibt eine Auswahl.
        If e.Index >= 0 And e.Index < Me.Items.Count Then
            Dim strItemCaption As String = Me.Items(e.Index).ToString()
            Try

                ' Font setzen und Abmasse ermitteln, 
                ' dann die Grsse des Items bestimmen.
                Dim fntCurrent As Font = _
                  New System.Drawing.Font( _
                    strItemCaption, _
                    m_sngFontSize, _
                    FontStyle.Regular, _
                    GraphicsUnit.Point)
                Dim MySizeF As SizeF = _
                  e.Graphics.MeasureString(strItemCaption, _
                    fntCurrent)

                ' Oben und unten jeweils ein Pixel Abstand.
                e.ItemHeight = CInt(MySizeF.Height) + 2
                e.ItemWidth = CInt(MySizeF.Width)
            Catch

                ' Hier mssen wir nichts machen, da 
                ' implizit die Prozedur verlassen wird.
            End Try
        End If
    End Sub

    Private Sub FontListBox_DrawItem( _
      ByVal sender As Object, _
      ByVal e As DrawItemEventArgs) _
      Handles MyBase.DrawItem

        ' Es gibt eine Auswahl.
        If e.Index >= 0 And e.Index < Me.Items.Count Then
            Dim strItemCaption As String = Me.Items(e.Index).ToString()
            Dim fntCurrent As Font
            Try

                ' Versuchen, den Font zu erstellen.
                fntCurrent = New System.Drawing.Font( _
                  strItemCaption, _
                  Me.FontSize, _
                  FontStyle.Regular, _
                  GraphicsUnit.Point)
            Catch
                Return
            End Try

            ' Das Item ist ausgewhlt.
            If (e.State And DrawItemState.Selected) = _
                DrawItemState.Selected Then

                ' Highlight-Rechteck fr das Element; dann 
                ' wird ein Fokusrechteck gezeichnet (dies
                ' knnte man so erweitern, dass das Zeichnen 
                ' nur erfolgt, wenn sich Windows im
                ' entsprechenden Modus befindet). Schliesslich 
                ' wird noch der Text gezeichnet.
                e.Graphics.FillRectangle(SystemBrushes.Highlight, e.Bounds)
                If Me.ShowFocusCues And m_blnHasFocus Then
                    ControlPaint.DrawFocusRectangle(e.Graphics, e.Bounds)
                End If
                e.Graphics.DrawString(strItemCaption, _
                  fntCurrent, _
                  SystemBrushes.HighlightText, _
                  e.Bounds.Left, _
                  e.Bounds.Top + 1)

            ' Das Item ist nicht ausgewhlt.
            Else

                ' Neuzeichnen des Hintergrunds, um alte States 
                ' zu entfernen, anschliessend zeichnen
                ' des Textes im entsprechenden Font.
                e.Graphics.FillRectangle(SystemBrushes.Window, e.Bounds)
                e.Graphics.DrawString( _
                  strItemCaption, _
                  fntCurrent, _
                  SystemBrushes.WindowText, _
                  e.Bounds.Left, _
                  e.Bounds.Top + 1)
            End If
        End If
    End Sub

    Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
        m_blnHasFocus = False
        Me.Invalidate()
    End Sub

    Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
        m_blnHasFocus = True
        Me.Invalidate()
    End Sub

    ' <summary>
    '   Leert die Liste und fllt sie mit den aktuellen Schriften neu.
    ' </summary>
    Public Sub Refill()
        Me.Items.Clear()
        MyBase.OnSelectedIndexChanged(New EventArgs())

        ' Falls gerade enumeriert wird, Enumeration abbrechen.
        If (Not m_EnumThread Is Nothing) AndAlso m_EnumThread.IsAlive Then
            m_EnumThread.Abort()
            m_blnEnumerating = False
        End If

        ' Thread fr EnumFonts erstellen und starten.
        m_EnumThread = New Threading.Thread(AddressOf EnumFonts)
        m_EnumThread.Start()
    End Sub

    ' <summary>
    '   Bricht einen laufenden Enumerationsvorgang ab.
    ' </summary>
    Public Sub AbortEnumeration()
        If (Not m_EnumThread Is Nothing) AndAlso m_EnumThread.IsAlive Then
            m_EnumThread.Abort()
            RaiseEvent EnumerationFinished(Me, _
              New EnumerationFinishedEventArgs( _
                True, _
                m_intValidFontsCount, _
                m_intInvalidFontsCount))
            m_blnEnumerating = False
        Else
            Throw New Exception("Es wird nicht enumeriert, " & _
              "daher kann auch nicht abgegrochen werden")
        End If
    End Sub

    ' <summary>
    '   Gibt die Grsse der Schrift an oder gibt diese zurck.
    ' </summary>
    ' <value>Grsse der Schrift, in der die Eintrge 
    '   angezeigt werden.</value>
    Public Property FontSize() As Single
        Get
            Return m_sngFontSize
        End Get
        Set(ByVal Value As Single)
            m_sngFontSize = Value
            Refill()
        End Set
    End Property

    ' <summary>
    '   Gibt an, ob gerade eine Enumeration durchgefhrt wird.
    ' </summary>
    ' <value>Wird gerade eine Enumeration durchgefhrt?</value>
    Public ReadOnly Property Enumerating() As Boolean
        Get
            Return m_blnEnumerating
        End Get
    End Property

    ' <summary>
    '   Enumeriert alle verfgbaren Fonts. Da dies 
    '   etwas mehr Zeit beanspruchen kann, erfolgt die
    '   Ausfhrung in einem eigenen Thread.
    ' </summary>
    Private Sub EnumFonts()
        m_blnEnumerating = True
        m_intInvalidFontsCount = 0
        m_intValidFontsCount = 0

        ' Alle Fontfamilien durchgehen.
        Dim fntfam As FontFamily
        For Each fntfam In System.Drawing.FontFamily.Families

            ' Font anhand des Namens erstellen.
            Try
                Dim fntCurrent As Font = _
                  New System.Drawing.Font( _
                    fntfam.Name, _
                    m_sngFontSize, _
                    FontStyle.Regular, _
                    GraphicsUnit.Pixel)
                Me.Items.Add(fntfam.Name)
                m_intValidFontsCount += 1
            Catch
                m_intInvalidFontsCount += 1
            End Try
        Next fntfam
        RaiseEvent EnumerationFinished(Me, _
          New EnumerationFinishedEventArgs( _
            False, m_intValidFontsCount, _
            m_intInvalidFontsCount))
        m_blnEnumerating = False
    End Sub
End Class